@@ -1,4 +1,4 @@
-#### This is from the R package
+#### This is originally from the R package
 ####
 ####  rrcov : Scalable Robust Estimators with High Breakdown Point
 ####
@@ -24,22 +24,28 @@
 
 ltsReg <- function(x, ...) UseMethod("ltsReg")
 
-ltsReg.formula <- function(formula, data, ...,
-			   model = TRUE, x.ret = FALSE, y.ret = FALSE)
+ltsReg.formula <- function(formula, data, subset, weights, na.action,
+			   model = TRUE, x.ret = FALSE, y.ret = FALSE,
+                           contrasts = NULL, offset, ...)
 {
+    cl <- match.call()
     ##	  method <- match.arg(method)
 
+    ## keep only the arguments which should go into the model frame
     mf <- match.call(expand.dots = FALSE)
-    mf$method <- mf$contrasts <- mf$model <- mf$x.ret <- mf$y.ret <- mf$... <-
-        NULL
+    m <- match(c("formula", "data", "subset", "weights", "na.action",
+                 "offset"), names(mf), 0)
+    mf <- mf[c(1, m)]
+    mf$drop.unused.levels <- TRUE
     mf[[1]] <- as.name("model.frame")
     mf <- eval.parent(mf)
     ##	  if (method == "model.frame") return(mf)
+
     mt <- attr(mf, "terms")
-    y <- model.extract(mf, "response")
+    y <- model.response(mf, "numeric") ## was model.extract(mf, "response")
 
     if (is.empty.model(mt)) { # "y ~ 0" : no coefficients
-	x <- NULL
+	x <- offset <- NULL
 	fit <- list(method = "ltsReg for empty model",
 		    coefficients = numeric(0), residuals = y,
 		    fitted.values = 0 * y, lts.wt = 1 + 0 * y,
@@ -48,6 +54,9 @@ ltsReg.formula <- function(formula, data, ...,
 	class(fit) <- "lts"
     }
     else {
+        w <- model.weights(mf)
+        offset <- model.offset(mf)
+
 	x <- model.matrix(mt, mf, contrasts)
 
 	## Check if there is an intercept in the model.
@@ -60,12 +69,16 @@ ltsReg.formula <- function(formula, data, ...,
 	fit <- ltsReg(x, y, intercept = (xint > 0), ...)
     }
 
-    fit$terms <- mt
-    fit$call <- match.call()
-    fit$contrasts <- attr(x, "contrasts")
-    fit$xlevels <- .getXlevels(mt, mf)
+    ## 3) return the na.action info
+    fit$na.action <- attr(mf, "na.action")
+    fit$offset <- offset
 
-    ##	  fit$na.action <- attr(mf, "na.action")
+    ## 4) return the contrasts used in fitting: possibly as saved earlier.
+    fit$contrasts <- attr(x, "contrasts")
+
+    fit$xlevels <- .getXlevels(mt, mf)
+    fit$call <- cl
+    fit$terms <- mt
 
     if(model) fit$model <- mf
     if(x.ret) fit$x <- x
@@ -229,7 +242,7 @@ ltsReg.default <-
 	    ans$coefficients <- reweighting$center
 	    ans$scale <- sqrt(sum(weights)/(sum(weights) - 1) * reweighting$cov)
 	    resid <- y - ans$coefficients
-	    ans$crit <- sum(sort((y - center)^2, quan)[1:quan])
+	    ans$crit <- sum(sort((y - center)^2, partial = quan)[1:quan])
 	    if (sum(weights) == n) {
 		cdelta.rew <- 1
 		correct.rew <- 1
@@ -365,7 +378,7 @@ ltsReg.default <-
 	    correct <- if(use.correction)
 		LTScnp2(p, intercept = intercept, n, alpha) else 1
 	    raw.cnp2[2] <- correct
-	    s0 <- sqrt((1/quan) * sum(sort(resid^2, quan)[1:quan]))
+	    s0 <- sqrt((1/quan) * sum(sort(resid^2, partial = quan)[1:quan]))
 	    sh0 <- s0
 	    qn.q <- qnorm((quan + n)/ (2 * n))
 	    s0 <- s0 / sqrt(1 - (2 * n)/(quan / qn.q) * dnorm(qn.q)) * correct
@@ -417,8 +430,8 @@ ltsReg.default <-
 		iR2 <- (sh0/sh)^2
 	    }
 	    else {
-		s1 <- sum(sort(resid^2, quan)[1:quan])
-		sh <- sum(sort(y^2, quan)[1:quan])
+		s1 <- sum(sort(resid^2, partial = quan)[1:quan])
+		sh <- sum(sort(y^2,     partial = quan)[1:quan])
 		iR2 <- s1/sh
 	    }
 
@@ -569,7 +582,10 @@ print.lts <- function (x, digits = max(3, getOption("digits") - 3), ...)
     invisible(x)
 }
 
-print.summary.lts <- function (x, digits = max(3, getOption("digits") - 3), ...)
+print.summary.lts <-
+    function(x, digits = max(3, getOption("digits") - 3),
+	     signif.stars = FALSE, ...)
+    ##			    ^^^^^ (since they are not quite correct ?)
 {
     cat("\nCall:\n",
 	paste(deparse(x$call), sep = "\n", collapse = "\n"), "\n\n", sep = "")
@@ -599,7 +615,8 @@ print.summary.lts <- function (x, digits = max(3, getOption("digits") - 3), ...)
 		" not defined because of singularities)\n", sep = "")
 	else
 	    cat("\nCoefficients:\n")
-	printCoefmat(x$coefficients, digits = digits, signif.stars = FALSE, na.print = "NA", ...)
+	printCoefmat(x$coefficients, digits = digits,
+		     signif.stars = signif.stars, ...)
     }
     else cat("\nNo coefficients\n")
 
